$PineTree::RunEnergy = 1;
$PineTree::DrainSpeed = 10;
$PineTree::TimeBeforeHeal = 2000;

datablock AudioProfile(PineTreeRustleSound)
{
	fileName = "./rustle.wav";
	description = AudioClose3D;
	preload = true;
};

datablock PlayerData(PlayerPineTreeArmor : PlayerStandardArmor)
{
	//airControl = 0.1;
	boundingBox = "5 5 10.6";
	cameraMaxDist = 8;
	cameraMinDist = 3.5; //0
	cameraVerticalOffset = 1;
	canJet = false;
	canRide = false;
	crouchBoundingBox = "5 5 10.6";
	firstPersonOnly = false;
	jumpForce = 400;
	maxBackwardCrouchSpeed = 2;
	maxBackwardSpeed = 2; //4
	maxDamage = 500;
	maxEnergy = 100;
	maxForwardCrouchSpeed = 3;
	maxForwardSpeed = 3; //7
	maxFreeLookAngle = 0; //3
	maxLookAngle = 0.75; //1.5708
	maxSideCrouchSpeed = 2;
	maxSideSpeed = 2; //6
	maxStepHeight = 0; //1
	maxUnderwaterBackwardSpeed = 3; //7.8
	maxUnderwaterForwardSpeed = 4; //8.4
	maxUnderwaterSideSpeed = 3; //7.8
	minLookAngle = -0.75; //-1.5708
	rechargeRate = 0.2;
	repairRate = 0.05;
	runForce = 4320;
	shapeFile = "./pineTree.dts";
	uiName = "Pine Tree";
	showEnergyBar = true;
};

datablock PlayerData(PlayerPineTreeNoChargeArmor : PlayerPineTreeArmor)
{
	maxBackwardCrouchSpeed = 4;
	maxBackwardSpeed = 4;
	maxForwardCrouchSpeed = 7;
	maxForwardSpeed = 7;
	maxSideCrouchSpeed = 6;
	maxSideSpeed = 6;
	maxUnderwaterBackwardSpeed = 7.8;
	maxUnderwaterForwardSpeed = 8.4;
	maxUnderwaterSideSpeed = 7.8;
	rechargeRate = 0;
	uiName = "";
};

function PlayerPineTreeArmor::onAdd(%data, %player)
{
	if(isObject(%client = %player.client) && %client.chestColor !$= "")
		%player.setNodeColor("ALL", %client.chestColor);
	else if(%player.chestColor !$= "")
		%player.setNodeColor("ALL", %player.chestColor);
	else
		%player.setNodeColor("ALL", "0 0.5 0.25 1");

	%player.lastXY = getWords(%player.getPosition(), 0, 1);
	%player.pineTreeFootstepLoop();
}

function player::pineTreeRunLoop(%player)
{
	cancel(%player.pineTreeRunSched);

	%data = %player.getDataBlock();

	if(%data.getName() !$= "PlayerPineTreeArmor" && %data.getName() !$= "PlayerPineTreeNoChargeArmor")
		return;

	%player.setEnergyLevel(%player.getEnergyLevel() - $PineTree::RunEnergy);

	if(%player.getEnergyLevel() >= $PineTree::RunEnergy)
		%player.pineTreeRunSched = %player.schedule($PineTree::DrainSpeed, pineTreeRunLoop);
	else
		%player.setDataBlock(PlayerPineTreeArmor);
}

function player::pineTreeFootstepLoop(%player)
{
	cancel(%player.pineTreeFootstepSched);

	%data = %player.getDataBlock();

	if(%data.getName() !$= "PlayerPineTreeArmor" && %data.getName() !$= "PlayerPineTreeNoChargeArmor")
		return;

	%pos = %player.getPosition();
	%offset = 2.2 * getWord(%player.getScale(), 2);
	%xyP = getWords(%pos, 0, 1);
	%vel = %player.getVelocity();
	%isGround = pineTreeGroundCheck(%player);

	if(%player.lastXY $= %xyP || !%isGround || %player.isInLiquid)
	{
		%player.pineTreeFootstepSched = %player.schedule(50, pineTreeFootstepLoop);

		return;
	}

	if(!isObject(%player.getMountedObject(0)))
	{
		if(mAbs(firstWord(%vel)) >= 0 && mAbs(getWord(%vel, 1)) >= 0) //If not moving slowly
		{
			if($Sim::Time - %player.lastPineTreeSound >= 1)
			{
				serverPlay3D(PineTreeRustleSound, vectorSub(%pos, "0 0" SPC %offset));
				%player.lastPineTreeSound = $Sim::Time;
			}
		}
	}

	%player.lastXY = %xyP;

	%player.pineTreeFootstepSched = %player.schedule(350, pineTreeFootstepLoop);
}

function pineTreeGroundCheck(%player)
{
	%pos0 = %player.getPosition();
	%offset = 2.2 * getWord(%player.getScale(), 2);
	%pos1 = vectorSub(%pos0, "0 0" SPC %offset);

	for(%i = 0; %i < 2; %i++) //Ensure he's not stuck in the ground, with 2 checks
	{
		initContainerBoxSearch(%pos[%i], "1.25 1.25 0.1", $TypeMasks::fxBrickObjectType | $Typemasks::TerrainObjectType);
		%col[%i] = containerSearchNext();
	}

	return isObject(%col0) && %col0 != %col1; //If on the ground, and ground is not overlapping him
}

package PlayerPineTree
{
	function armor::onNewDatablock(%data, %player)
	{
		if(%player.getDataBlock().getName() $= "PlayerPineTreeNoChargeArmor")
			%return = true;

		parent::onNewDatablock(%data, %player);

		if(%return || %player.getDataBlock().getName() !$= "PlayerPineTreeArmor")
			return;

		if(isObject(%client = %player.client) && %client.chestColor !$= "")
			%player.setNodeColor("ALL", %client.chestColor);
		else if(%player.chestColor !$= "")
			%player.setNodeColor("ALL", %player.chestColor);
		else
			%player.setNodeColor("ALL", "0 0.5 0.25 1");

		%player.lastXY = getWords(%player.getPosition(), 0, 1);
		%player.pineTreeFootstepLoop();
	}

	function serverCmdUpdateBodyColors(%client, %headColor, %hatColor, %accentColor, %packColor, %secondPackColor, %chestColor, %hipColor, %LLegColor, %RLegColor, %LArmColor, %RArmColor, %LHandColor, %RHandColor, %decalName, %faceName)
	{
		parent::serverCmdUpdateBodyColors(%client, %headColor, %hatColor, %accentColor, %packColor, %secondPackColor, %chestColor, %hipColor, %LLegColor, %RLegColor, %LArmColor, %RArmColor, %LHandColor, %RHandColor, %decalName, %faceName);

		if(isObject(%player = %client.player) && (%player.getDataBlock().getName() !$= "PlayerPineTreeArmor" || %player.getDataBlock().getName() $= "PlayerPineTreeNoChargeArmor"))
			%player.doTreeColors = true;
	}

	function gameConnection::applyBodyColors(%client)
	{
		parent::applyBodyColors(%client);

		if((isObject(%player = %client.player) && %player.getDataBlock().getName() !$= "PlayerPineTreeArmor") || (isFunction(%client.getClassName(), "setNodeColor") && (%client.getDataBlock().getClassName() !$= "PlayerPineTreeArmor" || %client.getDataBlock().getClassName() !$= "PlayerPineTreeNoChargeArmor")))
			return;

		if(isObject(%player))
			%obj = %player;
		else if(isFunction(%client.getClassName(), "setNodeColor")) //For bots.
			%obj = %client;

		if(!isObject(%obj) || !%obj.doTreeColors)
			return;

		%obj.doTreeColors = false;

		if(%client.chestColor !$= "")
			%obj.setNodeColor("ALL", %client.chestColor);
		else
			%obj.setNodeColor("ALL", "0 0.5 0.25 1");
	}

	function MiniGameSO::updateEnablePainting(%mini)
	{
		if(%mini.enablePainting)
			return parent::updateEnablePainting(%mini);

		for(%i = 0; %i < %mini.numMembers; %i++)
		{
			%client = %mini.member[%i];

			if(isObject(%player = %client.player) && (%player.getDataBlock().getName() $= "PlayerPineTreeArmor" || %player.getDataBlock().getName() $= "PlayerPineTreeNoChargeArmor"))
				commandToClient(%client, 'clientCmdSetPaintingDisabled', 0);
			else
				commandToClient(%client, 'clientCmdSetPaintingDisabled', 1);
		}

		return %mini;
	}

	function serverCmdUseSprayCan(%client, %colorID)
	{
		parent::serverCmdUseSprayCan(%client, %colorID);

		if(isObject(%player = %client.player) && (%player.getDataBlock().getName() $= "PlayerPineTreeArmor" || %player.getDataBlock().getName() $= "PlayerPineTreeNoChargeArmor") && isObject(%mini = %client.minigame) && !%mini.enablePainting)
			%player.currSprayCan = %colorID;
	}

	function PlayerPineTreeArmor::onDamage(%data, %player, %damage)
	{
		%parent = parent::onDamage(%data, %player, %damage);

		if(!%damage || %player.getState() $= "Dead")
			return %parent;

		%player.setRepairRate(0);

		cancel(%player.repairRateSched);
		%player.repairRateSched = %player.schedule($PineTree::TimeBeforeHeal, setRepairRate, PlayerPineTreeArmor.repairRate);

		return %parent;
	}

	function armor::onTrigger(%data, %player, %slot, %val)
	{
		if(%data.getName() !$= "PlayerPineTreeArmor" && %data.getName() !$= "PlayerPineTreeNoChargeArmor")
			return parent::onTrigger(%data, %player, %slot, %val);

		if(%slot == 3 && %val) //Crouch
		{
			%color = getWords(getColorIDTable(mClamp(%player.currSprayCan, 0, 65)), 0, 2) SPC 1;

			if(isObject(%client = %player.client))
			{
				%client.chestColor = %color;
				%client.applyBodyColors();
			}

			else
			{
				%player.chestColor = %color;
				GameConnection::applyBodyColors(%player);
			}
		}

		if(%slot == 4) //Jet
		{
			if(%val)
			{
				if(%player.getEnergyLevel() < $PineTree::RunEnergy)
					return;

				%player.setDataBlock(PlayerPineTreeNoChargeArmor);
				%player.pineTreeRunLoop();
			}

			else if(isEventPending(%player.pineTreeRunSched))
			{
				%player.setDataBlock(PlayerPineTreeArmor);

				cancel(%player.pineTreeRunSched);
			}
		}

		return parent::onTrigger(%data, %player, %slot, %val);
	}

	function serverCmdSit(%client)
	{
		if(!isObject(%player = %client.player) || (%player.getDataBlock().getName() !$= "PlayerPineTreeArmor" && %player.getDataBlock().getName() !$= "PlayerPineTreeNoChargeArmor"))
			return parent::serverCmdSit(%client);

		%offset = 2.2 * getWord(%player.getScale(), 2);
		%rotation = getWords(%player.getTransform(), 3, 6);

		%brick = new fxDTSBrick()
		{
			datablock = brickPineTreeData;
			isPlanted = 0;
			position = vectorAdd(%player.getPosition(), "0 0" SPC %offset);
			rotation = %rotation;
		};

		%plant = %brick.plant();

		if(%plant == 0)
			%player.setTransform(vectorSub(%brick.getPosition(), "0 0" SPC %offset) SPC %rotation);

		%brick.delete();

		return parent::serverCmdSit(%client);
	}

	function player::emote(%player, %image)
	{
		if((%player.getDatablock().getName() $= "PlayerPineTreeArmor" || %player.getDataBlock().getName() $= "PlayerPineTreeNoChargeArmor") && strLen(%image) >= 7 && (getSubStr(%image, 0, 7) $= "painLow" || getSubStr(%image, 0, 7) $= "painMid" || (strLen(%image) >= 8 &&  getSubStr(%image, 0, 8) $= "painHigh")))
			return 0;

		parent::emote(%player, %image);
	}

	function player::playPain(%player)
	{
		if((%player.getDatablock().getName() $= "PlayerPineTreeArmor" || %player.getDataBlock().getName() $= "PlayerPineTreeNoChargeArmor"))
			return 0;

		parent::playPain(%player);
	}

	function armor::onEnterLiquid(%data, %obj, %coverage, %type)
	{
		parent::onEnterLiquid(%data, %obj, %coverage, %type);

		%obj.isInLiquird = true;
	}
	function armor::onLeaveLiquid(%data, %obj, %coverage, %type)
	{
		parent::onLeaveLiquid(%data, %obj, %coverage, %type);

		%obj.isInLiquid = false;
	}
};
activatePackage(PlayerPineTree);